<?php

//  Copyright (C) 2002-2006 Balint Kis (balint@k-i-s.net)
//  Copyright (C) 2005-2007 Solomon Peachy (pizza@shaftnet.org)

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

include_once "config.php";
include_once "mime.php";
include_once "exif.php";
include_once "iptc.php";
include_once "rdf.php";
include_once "site.php";

function count_storage_space_by_user($database, $user_id) {
  $file_sizes = pg_query($database, "
     select sum(filesize) 
       from files, photo_version, photo
      where photo.users = $user_id
       and photo.identifier = photo_version.photo
       and files.version = photo_version.identifier");

  $size = pg_fetch_row($file_sizes);

  return $size[0];
}

function dirsize($database, $directory) {
  $result = array("size" => 0, "files" => 0);

  $query = pg_query($database, "select count(identifier), sum(filesize) from files where path LIKE '$directory%'");

  if ($query && pg_num_rows($query)) {
    $row = pg_fetch_row($query);
    $result['files'] = $row[0];
    $result['size'] = $row[1];
  }

  return $result;
}

function get_current_volume($database, $image_file) {
  global $image_repository_path;
  global $po_options_default;

  $curr_volume = readlink($image_repository_path."/current");
  $new_volume = $curr_volume;

  if (substr($curr_volume, 0, 1) == '/') {
    $curr_volume = substr($curr_volume, strlen($image_repository_path));
  }

  do {
    $dir_size = dirsize($database, $new_volume);

    if (is_file($image_file)) {
      $image_size = filesize($image_file);
    } else {
      $image_size = 0;
    }

    $dir_size['size'] = round($dir_size['size']/1048576, 2);

    /* if size is smaller return the volume as current volume */
    if (($dir_size['size'] + $image_size/1048576) < $po_options_default['volume_max_size'])
      break;

    $new_volume++;
    if (($directory_handle = @opendir($image_repository_path."/".$new_volume))) {
      /* if the next volume check if there is enough space */
      closedir($directory_handle);
      continue;
    } else {
      /* if the next volume does not exist, try to create it */
      if (!mkdir($image_repository_path."/".$new_volume, 0700))
        return FALSE;
      if (!mkdir($image_repository_path."/".$new_volume."/00000", 0700))
        return FALSE;
      if (!mkdir($image_repository_path."/".$new_volume."/00001", 0700))
        return FALSE;
      if (!mkdir($image_repository_path."/".$new_volume."/00002", 0700))
        return FALSE;
      break;
    }
  } while (TRUE);

  if ($new_volume != $curr_volume) {
    unlink($image_repository_path."/current");
    po_symlink($new_volume, "$image_repository_path/current");
  }

  /* Make sure we pass back an absolute path */
  if (substr($new_volume, 0, 1) != '/') {
    $new_volume = "$image_repository_path/$new_volume";
  }

  return $new_volume;
}

function display_data_decoding_radio() {
  global $strings;

  print "<table id=\"metadata\">\n";
  print "<tr>";
  print emit_th("&nbsp;");
  print emit_th($strings['generic']['skip']);
  print emit_th($strings['generic']['first']);
  print emit_th($strings['generic']['second']);
  print emit_th($strings['generic']['third']);
  print "</tr>";
  print "<tr>";
  print emit_td("EXIF");
  print emit_td("<input type=\"radio\" name=\"decode_exif\" onclick=\"disabler1();\" value=\"0\" />");
  print emit_td("<input type=\"radio\" name=\"decode_exif\" onclick=\"disabler1();\" value=\"1\" checked=\"checked\" />");
  print emit_td("<input type=\"radio\" name=\"decode_exif\" onclick=\"disabler1();\" value=\"2\" />");
  print emit_td("<input type=\"radio\" name=\"decode_exif\" onclick=\"disabler1();\" value=\"3\" />");
  print "</tr>";
  print "<tr>";
  print emit_td("XMP");
  print emit_td("<input type=\"radio\" name=\"decode_rdf\" onclick=\"disabler3();\" value=\"0\" />");
  print emit_td("<input type=\"radio\" name=\"decode_rdf\" onclick=\"disabler3();\" value=\"1\" />");
  print emit_td("<input type=\"radio\" name=\"decode_rdf\" onclick=\"disabler3();\" value=\"2\" checked=\"checked\" />");
  print emit_td("<input type=\"radio\" name=\"decode_rdf\" onclick=\"disabler3();\" value=\"3\" />");
  print "</tr>";
  print "<tr>";
  print emit_td("IPTC");
  print emit_td("<input type=\"radio\" name=\"decode_iptc\" onclick=\"disabler2();\" value=\"0\" />");
  print emit_td("<input type=\"radio\" name=\"decode_iptc\" onclick=\"disabler2();\" value=\"1\" />");
  print emit_td("<input type=\"radio\" name=\"decode_iptc\" onclick=\"disabler2();\" value=\"2\" />");
  print emit_td("<input type=\"radio\" name=\"decode_iptc\" onclick=\"disabler2();\" value=\"3\" checked=\"checked\" />");
  print "</tr>";
  print "</table>\n";
}

function extract_exiftool_metadata($image_data, $index, $type = "EXIF") {
  global $sys_exiftool;
  global $strings;

  $image_data["exiftool_data_$type"] = NULL;
  $exif_data = array();

  $exclude_args = "";

  switch ($type) {
  case 'EXIF' :
    $exclude_args = "-x xmp:all -x iptc:all";
    break;
  case 'IPTC' :
    $exclude_args = "-iptc:all";
    break;
  case 'XMP' :
    $exclude_args = "-xmp:all";
    break;
  case 'ORIENT' :
    $exclude_args = "-Orientation -ICCProfile -ColorSpace -ProfileClass -Make -Model";
    break;
  }

  print "<li>".$strings['import']['extracting_metadata']." ($type) ... ";

  if (!is_executable($sys_exiftool)) {
    print err_str($strings['errors']['exiftool_not_installed']) ."</li>";
    return $image_data;
  }

  $handle = popen("$sys_exiftool $exclude_args " . $image_data['file'][$index]['name_tmp'], "r");

  if ($handle === FALSE) {
    print err_str($strings['errors']['exiftool_failed']) ."</li>";
    return $image_data;
  }

  while (!feof($handle)) {
    $line = fgets($handle);
    $parts = explode(":", $line, 2);
    $parts[0] = trim($parts[0]);
    if (!isset($parts[1])) continue;
    $parts[1] = trim($parts[1]);
    if ($parts[0] == "File Name") continue;
    if ($parts[0] == "Directory") continue;
    if (substr($parts[1], 2, 6) == "Binary") continue;
    //  print "<li>key: $parts[0] value: $parts[1]</li>";
    $exif_data[$parts[0]] = $parts[1];
  }
  pclose($handle);
  $image_data["exiftool_data_$type"] = $exif_data;

  print $strings['generic']['done']."</li>";

  return $image_data;
}

function transfer_metadata($image_data, $source_name, $dest_name, $verbose, $fix_orient) {
  global $sys_exiftool;
  global $po_options;
  global $strings;

  $retval = FALSE;

  // image_data can be NULL!

  /* Copy over image tags */
  if (is_executable($sys_exiftool)) {
    if ($verbose)
      print "<li>".$strings['import']['transferring_metadata']." ... ";
    
    if ($fix_orient) {
      $orient = " -Orientation=1 -n ";
    } else {
      $orient = "";
    }

    $cmdline = "$sys_exiftool -q -overwrite_original -TagsFromFile " . escapeshellarg($source_name) ." $orient ". escapeshellarg($dest_name);
    system($cmdline, $retval);

    if (!$retval) {
      if ($verbose) {
	print $strings['generic']['done'].".</li>";
      }
    } else {
      if ($verbose) {
	print "<font color=\"red\">".$strings['generic']['failed'].".</font></li>";
	print "<li><pre>$cmdline</pre></li>";
      }
    }
  }
  return !$retval;
}

function photo_import($database, $index, $user_id, $default_folder_id, &$image_data) {
  global $mime_type;
  global $po_options;
  global $strings;
  global $image_repository_path;

  if (($current_volume = get_current_volume($database, $image_data['file'][$index]['name_tmp'])) == FALSE) {
    print err_str($strings['errors']['failed_volume']);
    return FALSE;
  }

  if (!is_readable($image_data['file'][$index]['name_tmp'])) {
    print err_str($strings['errors']['invalid_file'] . "(" . $image_data['file'][$index]['name'] .")");
    return FALSE;
  }

  /* Trims the temporary directory name that is used during bulk upload */
  $pos = strpos($image_data['file'][$index]['name'], "/");
  if (is_integer($pos)) {
    $original_name = substr($image_data['file'][$index]['name'], strrpos($image_data['file'][$index]['name'], "/") + 1);
  } else {
    $original_name = $image_data['file'][$index]['name'];
  }

  if ($original_name == '') {
    print err_str($strings['errors']['invalid_file'] . "(" . $image_data['file'][$index]['name'] .")");
    return FALSE;
  }

  pg_query($database, "begin");

  $original_file_type                 = strtolower($image_data['file'][$index]['type']);

  $next_index = pg_fetch_row(pg_query($database, "select nextval('photo_id_sequence')"));

  $hires_name = $current_volume . "/00000/" . $next_index[0] . "_3_orig." . $original_file_type;

  $image_data['file'][$index]['original_name'] = $original_name;
  $multi_page_parameter               = $mime_type[$original_file_type]['page'];
  $mime_type_name                     = $mime_type[$original_file_type]['name'];
  $decoder                            = $mime_type[$original_file_type]['decoder'];

  if (!isset($image_data['folder']))
    $image_data['folder'] = $default_folder_id;

  $thumb_name = $current_volume . "/00002/" . $next_index[0] . "_1." . $po_options['thumb_format'];
  $preview_name = $current_volume . "/00001/" . $next_index[0] . "_2." . $po_options['preview_format'];

  $target_folder = pg_fetch_row(pg_query($database, "select caption, users from folder where identifier = $image_data[folder]"));
  if ($target_folder[1] != $user_id) {
    print err_str($strings['errors']['not_folder_owner']." (".emit_a("folder.php?folder=$image_data[folder]", $target_folder[0]). ")");
    $image_data['folder'] = $default_folder_id;
    $target_folder = pg_fetch_row(pg_query($database, "select caption from folder where identifier = $image_data[folder]"));
  }

  print "<p>".sprintf($strings['import']['importing_into_folder'], $original_name, $mime_type_name, emit_a("folder.php?folder=$image_data[folder]", "<strong>$target_folder[0]</strong>"))."</p>";

  flush();
  print "<ul class=\"import\">";

  /* Check quotas */
  if ($po_options['quota_size'] > 0) {
    $used_storage_space = count_storage_space_by_user($database, $user_id);
    $used_storage_space += filesize($image_data['file'][$index]['name_tmp']);
    if ($po_options['quota_size'] <= $used_storage_space) {
      pg_query($database, "rollback");
      $err = error_reporting(0);
      unlink($image_data['file'][$index]['name_tmp']);
      error_reporting($err);
      $po_options['quota_size'] /= 1048576;
      print "<li>";
      print err_str($strings['errors']['quota_space_full'] . " ($po_options[quota_size] MB)");
      print $strings['import']['no_more_uploads'];
      print "</li>";
      print "</ul>";
      return FALSE;
    }
  }

  if ($po_options['quota_count'] > 0) {
    $num_of_photo_versions = pg_fetch_row(pg_query($database, "select count_photo_versions_by_user($user_id)"));
    $num_of_photo_versions++;
    if ($po_options['quota_count'] <= $num_of_photo_versions[0]) {
      pg_query($database, "rollback");
      $err = error_reporting(0);
      unlink($image_data['file'][$index]['name_tmp']);
      error_reporting($err);
      print "<li>";
      print err_str($strings['errors']['quota_count_full'] . " ($po_options[quota_count] ".$strings['generic']['files'] .")");
      print $strings['import']['no_more_uploads'];
      print "</li>";
      print "</ul>";
      return FALSE;
    }
  }

  /* Extract any metadata */
  if ($image_data['decode_exif'] == 1)
    $image_data = photo_parse_exif($database, $index, $user_id, $image_data);
  else if ($image_data['decode_iptc'] == 1)
    $image_data = photo_parse_iptc($database, $index, $user_id, $image_data);
  else if ($image_data['decode_rdf'] == 1)
    $image_data = photo_parse_rdf($database, $index, $user_id, $image_data);

  if ($image_data['decode_exif'] == 2)
    $image_data = photo_parse_exif($database, $index, $user_id, $image_data);
  else if ($image_data['decode_iptc'] == 2)
    $image_data = photo_parse_iptc($database, $index, $user_id, $image_data);
  else if ($image_data['decode_rdf'] == 2)
    $image_data = photo_parse_rdf($database, $index, $user_id, $image_data);

  if ($image_data['decode_exif'] == 3)
    $image_data = photo_parse_exif($database, $index, $user_id, $image_data);
  else if ($image_data['decode_iptc'] == 3)
    $image_data = photo_parse_iptc($database, $index, $user_id, $image_data);
  else if ($image_data['decode_rdf'] == 3)
    $image_data = photo_parse_rdf($database, $index, $user_id, $image_data);

  /* Figure out orientation and necessary transforms */
  $orientation = orientation_id_from_string($database, $image_data["orientation"]); 
  $image_data["orientation_xform"] = orientation_to_xform($database, $orientation);

  if (!file_upload($image_data, $index, $hires_name, $preview_name, $thumb_name, $multi_page_parameter, $decoder, $database)) {
    pg_query($database, "rollback");
    print "<li>".$strings['import']['cleaning_up']."</li>";
    $err = error_reporting(0);
    unlink($hires_name);
    unlink($preview_name);
    unlink($thumb_name);
    error_reporting($err);
    print "</ul>";
    return FALSE;
  }

  /* Sanity-check the timestamp */
  $image_data['date_of_exposure'] = check_date_validity($image_data['date_of_exposure']);

  /* Insert into database */
  $query = "insert into photo
            (identifier, users, folder, location,
             caption, date_of_exposure, access_rights, copyright_statement,
             views, hide_original, author, title,
             caption_writer, category, credit, source,
             headline, instructions, transmission_reference,
             supplemental_category, web_statement, store_url, comments)
     values ($next_index[0], '$user_id', '$image_data[folder]', $image_data[location],
             '$image_data[caption]', $image_data[date_of_exposure], '$image_data[access_rights]', '$image_data[copyright]',
             0, '$image_data[hide_original]', '$image_data[author]', '$image_data[title]', 
             '$image_data[caption_writer]', '$image_data[category]', '$image_data[credit]', '$image_data[source]',
             '$image_data[headline]', '$image_data[instructions]', '$image_data[transmission_reference]',
             '$image_data[supplemental_category]', '$image_data[web_statement]', '$image_data[store_url]', '$image_data[remark]')";
  $result = pg_query($database, $query);
  if (!$result) {

    $error = pg_last_error($database);
    print "<li><font color=\"red\">$error</font></li>";

    pg_query($database, "rollback");
    print "<li>" . err_str($strings['errors']['db_insert_failed']) ."</li>";
    print "<li><pre>$query</pre></li>\n";

    print "<li>".$strings['import']['cleaning_up']."</li>";
    $err = error_reporting(0);
    unlink($hires_name);
    unlink($preview_name);
    unlink($thumb_name);
    error_reporting($err);
    print "</ul>";
    return FALSE;
  }

  /* Now for the keywords */
  foreach ($image_data['keywords'] as $keyword) {
    $keyword = strtolower(trim($keyword));
    $keyword = trim($keyword, ','); /* Trailing commas */
    $keyword = trim($keyword);
    if ($keyword == "") continue;
    $keyword = pg_escape_string($keyword);
    
    $query = "insert into photo_keywords (photo, keyword) 
                   values ($next_index[0], '$keyword')";
    $result = pg_query($database, $query);

    if (!$result) {
      pg_query($database, "rollback");
      print "<li>" . err_str($strings['errors']['db_insert_failed']) ."</li>";
      print "<li><pre>$query</pre></li>\n";
      print "<li>".$strings['import']['cleaning_up']."</li>";
      $err = error_reporting(0);
      unlink($hires_name);
      unlink($preview_name);
      unlink($thumb_name);
      error_reporting($err);
      print "</ul>";
      return FALSE;
    }
  }

  $comment = $image_data['file'][$index]['remark'];

  $next_version = pg_fetch_row(pg_query($database, "select nextval('photo_version_id_sequence')"));

  $orientation = orientation_id_from_string($database, $image_data["orientation"]); 

  $query = "insert into photo_version(identifier, key, photo, master, original_image_name, comment, colorspace, orientation)
                                   values ($next_version[0], 1,  $next_index[0], 't', '$original_name', '$comment', " . $image_data['colorspace'] . " , $orientation )";

  $result = pg_query($database, $query);

  if (!$result) {
    pg_query($database, "rollback");
    print "<li>" . err_str($strings['errors']['db_insert_failed']) ."</li>";
    print "<li><pre>$query</pre></li>\n";
    print "<li>".$strings['import']['cleaning_up']."</li>";
    $err = error_reporting(0);
    unlink($hires_name);
    unlink($preview_name);
    unlink($thumb_name);
    error_reporting($err);
    print "</ul>";
    return FALSE;
  }

  /* Thumbnail */
  $image_dimension = po_get_image_size($thumb_name);
  $size = filesize($thumb_name);
  $file = substr($thumb_name, strlen($image_repository_path));
  $query = "INSERT INTO files 
     (identifier, version, size, x_res, y_res, path, filesize, params, comments, watermark) 
      VALUES ((select nextval('files_id_sequence')), $next_version[0], 1, 
              $image_dimension[0], $image_dimension[1], '$file', 
              $size, '', '', null)";
  $result = pg_query($database, $query);

  if (!$result) {
    pg_query($database, "rollback");
    print "<li>" . err_str($strings['errors']['db_insert_failed']) ."</li>";
    print "<li><pre>$query</pre></li>\n";
    print "<li>".$strings['import']['cleaning_up']."</li>";
    $err = error_reporting(0);
    unlink($hires_name);
    unlink($preview_name);
    unlink($thumb_name);
    error_reporting($err);
    print "</ul>";
    return FALSE;
  }

  /* preview image */

  if ($po_options['watermark_location'] &&
      $po_options['watermark_brightness'] &&
      $po_options['watermark_photo']) {
    $watermark = $po_options['watermark_photo'];
  } else {
    $watermark = "null";
  }

  $image_dimension = po_get_image_size($preview_name);
  $size = filesize($preview_name);
  $file = substr($preview_name, strlen($image_repository_path));
  $query = "INSERT INTO files 
     (identifier, version, size, x_res, y_res, path, filesize, params, comments, watermark) 
      VALUES ((select nextval('files_id_sequence')), $next_version[0], 2, 
              $image_dimension[0], $image_dimension[1], '$file', 
              $size, '', '', $watermark)";
  $result = pg_query($database, $query);

  if (!$result) {
    pg_query($database, "rollback");
    print "<li>" . err_str($strings['errors']['db_insert_failed']) ."</li>";
    print "<li><pre>$query</pre></li>\n";
    print "<li>".$strings['import']['cleaning_up']."</li>";
    $err = error_reporting(0);
    unlink($hires_name);
    unlink($preview_name);
    unlink($thumb_name);
    error_reporting($err);
    print "</ul>";
    return FALSE;
  }

  /* original image */
  $image_dimension = po_get_image_size($hires_name);
  $size = filesize($hires_name);
  $file = substr($hires_name, strlen($image_repository_path));
  $query = "INSERT INTO files 
     (identifier, version, size, x_res, y_res, path, filesize, params, comments, watermark) 
      VALUES ((select nextval('files_id_sequence')), $next_version[0], 3, 
              $image_dimension[0], $image_dimension[1], '$file', 
              $size, '', '', null)";
  $result = pg_query($database, $query);

  if (!$result) {
    pg_query($database, "rollback");
    print "<li>" . err_str($strings['errors']['db_insert_failed']) ."</li>";
    print "<li><pre>$query</pre></li>\n";
    print "<li>".$strings['import']['cleaning_up']."</li>";
    $err = error_reporting(0);
    unlink($hires_name);
    unlink($preview_name);
    unlink($thumb_name);
    error_reporting($err);
    print "</ul>";
    return FALSE;
  }

  if ($image_data['storage_location_type'] == "null")
    $image_data['storage_location_type'] = "";

  $storage_location = $image_data['storage_location_type'].$image_data['storage_location_id'];
  $query = "insert into photo_dupe (identifier, key, photo, date_of_creation, storage_location, storage_index, remark, version)
                                   values (nextval('photo_dupe_id_sequence'), 1, $next_index[0], now(), '$storage_location', '$image_data[storage_index]', 'Original', $next_version[0])";
  $result = pg_query($database, $query);

  if (!$result) {
    pg_query($database, "rollback");
    print "<li>" . err_str($strings['errors']['db_insert_failed']) ."</li>";
    print "<li><pre>$query</pre></li>\n";
    print "<li>".$strings['import']['cleaning_up']."</li>";
    $err = error_reporting(0);
    unlink($hires_name);
    unlink($preview_name);
    unlink($thumb_name);
    error_reporting($err);
    print "</ul>";
    return FALSE;
  }
	
  if ($image_data['latitude'] == "") $image_data['latitude'] = "null";
  if ($image_data['longitude'] == "") $image_data['longitude'] = "null";
  if ($image_data['altitude'] == "") $image_data['altitude'] = "null";
  if ($image_data['direction'] == "") $image_data['direction'] = "null";

  $query = "insert into photo_tech (identifier, photo, camera, lens, film, flash, scanner, filter, scan_resolution, aperture, focal_length, shutter, camera_metering, camera_program, flash_mode, exposure_comp, flash_comp, ev_difference, iso_override, support, scan_bitdepth, scan_multiscan, exif, iptc, latitude, longitude, altitude, img_direction, rdf)
        values (nextval('photo_tech_id_sequence'), $next_index[0], $image_data[camera], $image_data[lens],
                $image_data[film], $image_data[flash], $image_data[scanner],
                $image_data[filter], $image_data[scan_resolution], $image_data[aperture],
                $image_data[focal_length], $image_data[shutter], $image_data[camera_metering],
                $image_data[camera_program], $image_data[flash_mode], $image_data[exp_comp],
                $image_data[flash_comp], $image_data[exp_diff], $image_data[iso_override],
                $image_data[support], $image_data[scan_bitdepth], $image_data[scan_multiscan], 
                '$image_data[exif]', '$image_data[iptc]', $image_data[latitude], $image_data[longitude], $image_data[altitude], $image_data[direction], '$image_data[rdf]')";
  $result = pg_query($database, $query);
  if (!$result) {

    $error = pg_last_error($database);
    print "<li><font color=\"red\">$error</font></li>";
    
    pg_query($database, "rollback");
    print "<li>" . err_str($strings['errors']['db_insert_failed']) ."</li>";
    print "<li><pre>$query</pre></li>\n";
    print "<li>".$strings['import']['cleaning_up']."</li>";
    $err = error_reporting(0);
    unlink($hires_name);
    unlink($preview_name);
    unlink($thumb_name);
    error_reporting($err);
    print "</ul>";
    return FALSE;
  }

  print "<li>".$strings['import']['complete']." ( ". emit_a("photo.php?photo=$next_index[0]", "#" . $next_index[0]) .")</li>";
  pg_query($database, "commit");
  print "</ul>";

  /* If we have an embedded preview image, import it as a secondary version */
  if ($image_data['jpgfromraw']) {
    $image_data_b = $image_data;

    /* Transfer metadata over to preview image */
    transfer_metadata($image_data_b, 
		      $hires_name, 
		      $image_data['jpgfromraw'], FALSE, FALSE);

    /* Fill in the blanks */
    $image_data_b['file'][$index]['name']     = $image_data['file'][$index]['name'] . "_jpgfromraw.jpg";
    $image_data_b['file'][$index]['original_name']     = $image_data['file'][$index]['name'] . "_jpgfromraw.jpg";
    $image_data_b['file'][$index]['name_tmp'] = $image_data['jpgfromraw'];
    $image_data_b['file'][$index]['type']     = "jpeg";
    $image_data_b['file'][$index]['remark']   = "";
    $image_data_b['file'][$index]['master']   = "f";
    $image_data_b['orientation'] = $image_data['orientation_orig'];
    $image_data_b['file'][$index]['remark']   = "Preview image extracted from RAW";

    /* And import it.  This will unlink the temp file */
    photo_version_import($database, $index, $user_id, $next_index[0], $image_data_b);
  }
  
  return $next_index[0];
}

function photo_version_import($database, $index, $user_id, $photo_id, $image_data) {
  global $mime_type;
  global $po_options;
  global $strings;
  global $image_repository_path;

  if (($current_volume = get_current_volume($database, $image_data['file'][$index]['name_tmp'])) == FALSE) {
    print err_str($strings['errors']['failed_volume']);
    return FALSE;
  }

  if (!is_readable( $image_data['file'][$index]['name_tmp'])) {
    print err_str($strings['errors']['invalid_file'] . "(" . $image_data['file'][$index]['name'] .")");
    return FALSE;
  }

  /* Trims the temporary directory name that is used during bulk upload */

  $pos = strpos($image_data['file'][$index]['name'], "/");
  if (is_integer($pos)) {
    $original_name = substr($image_data['file'][$index]['name'], strrpos($image_data['file'][$index]['name'], "/") + 1);
  } else {
    $original_name = $image_data['file'][$index]['name'];
  }

  if ($original_name == '') {
    print err_str($strings['errors']['invalid_file'] . "(" . $image_data['file'][$index]['name'] .")");
    return FALSE;
  }

  pg_query($database, "begin");

  $original_file_type                 = strtolower($image_data['file'][$index]['type']);

  $next_index = pg_fetch_row(pg_query($database, "select nextval('photo_id_sequence')"));

  $hires_name = $current_volume . "/00000/" . $next_index[0] . "_3_orig." . $original_file_type;

  $multi_page_parameter               = $mime_type[$original_file_type]['page'];
  $mime_type_name                     = $mime_type[$original_file_type]['name'];
  $decoder                            = $mime_type[$original_file_type]['decoder'];

  $thumb_name = $current_volume . "/00002/" . $next_index[0] . "_1." . $po_options['thumb_format'];
  $preview_name = $current_volume . "/00001/" . $next_index[0] . "_2." . $po_options['preview_format'];

  print "<p>". sprintf($strings['import']['importing_new_version'], $original_name, $mime_type_name, emit_a("photo.php?photo=$photo_id&detail_info=5", "<strong>$photo_id</strong>")) ."</p>";

  flush();
  print "<ul class=\"import\">";

  /* Check quotas */  
  if ($po_options['quota_size'] > 0) {
    $used_storage_space = count_storage_space_by_user($database, $user_id);
    $used_storage_space += filesize($image_data['file'][$index]['name_tmp']);
    if ($po_options['quota_size'] <= $used_storage_space) {
      pg_query($database, "rollback");
      $err = error_reporting(0);
      unlink($image_data['file'][$index]['name_tmp']);
      error_reporting($err);
      $po_options['quota_size'] /= 1048576;
      print "<li>";
      print err_str($strings['errors']['quota_space_full'] . " ($po_options[quota_size] MB)");
      print $strings['import']['no_more_uploads'];
      print "</li>";
      print "</ul>";
      return FALSE;
    }
  }

  if ($po_options['quota_count'] > 0) {
    $num_of_photo_versions = pg_fetch_row(pg_query($database, "select count_photo_versions_by_user($user_id)"));
    $num_of_photo_versions++;
    if ($po_options['quota_count'] <= $num_of_photo_versions[0]) {
      pg_query($database, "rollback");
      $err = error_reporting(0);
      unlink($image_data['file'][$index]['name_tmp']);
      error_reporting($err);
      print "<li>";
      print err_str($strings['errors']['quota_count_full'] . " ($po_options[quota_count] ".$strings['generic']['files'] .")");
      print $strings['import']['no_more_uploads'];
      print "</li>";
      print "</ul>";
      return FALSE;
    }
  }

  /* Orientation and colorspace */
  $image_data = extract_exiftool_metadata($image_data, $index, "ORIENT");
  $image_data = photo_parse_exif_orientation_colorspace($database, $index, $user_id, $image_data, "ORIENT");

  $orientation = orientation_id_from_string($database, $image_data['orientation']);
  $colorspace = $image_data['colorspace'] == "" ? "1" : $image_data['colorspace'];

  $master = $image_data['file'][$index]['master'];
  if (!file_upload($image_data, $index, $hires_name, $preview_name, $thumb_name, $multi_page_parameter, $decoder, $database)) {
    pg_query($database, "rollback");
    print "<li>".$strings['import']['cleaning_up']."</li>";
    $err = error_reporting(0);
    unlink($hires_name);
    unlink($preview_name);
    unlink($thumb_name);
    error_reporting($err);
    print "</ul>";
    return FALSE;
  }

  if ($master == 't') {
    $query = "update photo_version set master='f' where photo=$photo_id";
    $result = pg_query($database, $query);
    if (!$result) {
      pg_query($database, "rollback");
      print "<li>" . err_str($strings['errors']['db_insert_failed']) ."</li>";
      print "<li><pre>$query</pre></li>\n";
      print "<li>".$strings['import']['cleaning_up']."</li>";
      $err = error_reporting(0);
      unlink($hires_name);
      unlink($preview_name);
      unlink($thumb_name);
      error_reporting($err);
      print "</ul>";
      return FALSE;
    }
  }

  $key = pg_fetch_row(pg_query($database, "select max(key) from photo_version where photo=$photo_id"));
  $key[0]++;

  $comment = $image_data['file'][$index]['remark'];
  $pos = strpos($image_data['file'][$index]['name'], "/");
  if (is_integer($pos)) {
    $original_name = substr($image_data['file'][$index]['name'], strrpos($image_data['file'][$index]['name'], "/") + 1);
  } else {
    $original_name = $image_data['file'][$index]['name'];
  }

  $next_version = pg_fetch_row(pg_query($database, "select nextval('photo_version_id_sequence')"));

  $query = "insert into photo_version (identifier, key, photo, master, original_image_name, comment, colorspace, orientation)
            values ($next_version[0], '$key[0]', '$photo_id', '$master', '$original_name', '$comment', $colorspace, $orientation)";
  $result = pg_query($database, $query);
  if (!$result) {
    pg_query($database, "rollback");
    print "<li>" . err_str($strings['errors']['db_insert_failed']) ."</li>";
    print "<li><pre>$query</pre></li>\n";
    print "<li>".$strings['import']['cleaning_up']."</li>";
    $err = error_reporting(0);
    unlink($hires_name);
    unlink($preview_name);
    unlink($thumb_name);
    error_reporting($err);
    print "</ul>";
    return FALSE;
  }

  /* Thumbnail */
  $image_dimension = po_get_image_size($thumb_name);
  $size = filesize($thumb_name);
  $file = substr($thumb_name, strlen($image_repository_path));
  $query = "INSERT INTO files 
     (identifier, version, size, x_res, y_res, path, filesize, params, comments, watermark) 
      VALUES ((select nextval('files_id_sequence')), $next_version[0], 1, 
              $image_dimension[0], $image_dimension[1], '$file', 
              $size, '', '', null)";
  $result = pg_query($database, $query);

  if (!$result) {
    pg_query($database, "rollback");
    print "<li>" . err_str($strings['errors']['db_insert_failed']) ."</li>";
    print "<li><pre>$query</pre></li>\n";
    print "<li>".$strings['import']['cleaning_up']."</li>";
    $err = error_reporting(0);
    unlink($hires_name);
    unlink($preview_name);
    unlink($thumb_name);
    error_reporting($err);
    print "</ul>";
    return FALSE;
  }

  /* preview image */
  if ($po_options['watermark_location'] &&
      $po_options['watermark_brightness'] &&
      $po_options['watermark_photo']) {
    $watermark = $po_options['watermark_photo'];
  } else {
    $watermark = "null";
  }

  $image_dimension = po_get_image_size($preview_name);
  $size = filesize($preview_name);
  $file = substr($preview_name, strlen($image_repository_path));
  $query = "INSERT INTO files 
     (identifier, version, size, x_res, y_res, path, filesize, params, comments, watermark) 
      VALUES ((select nextval('files_id_sequence')), $next_version[0], 2, 
              $image_dimension[0], $image_dimension[1], '$file', 
              $size, '', '', $watermark)";
  $result = pg_query($database, $query);

  if (!$result) {
    pg_query($database, "rollback");
    print "<li>" . err_str($strings['errors']['db_insert_failed']) ."</li>";
    print "<li><pre>$query</pre></li>\n";
    print "<li>".$strings['import']['cleaning_up']."</li>";
    $err = error_reporting(0);
    unlink($hires_name);
    unlink($preview_name);
    unlink($thumb_name);
    error_reporting($err);
    print "</ul>";
    return FALSE;
  }

  /* original image */
  $image_dimension = po_get_image_size($hires_name);
  $size = filesize($hires_name);
  $file = substr($hires_name, strlen($image_repository_path));
  $query = "INSERT INTO files 
     (identifier, version, size, x_res, y_res, path, filesize, params, comments, watermark) 
      VALUES ((select nextval('files_id_sequence')), $next_version[0], 3, 
              $image_dimension[0], $image_dimension[1], '$file', 
              $size, '', '', null)";
  $result = pg_query($database, $query);

  if (!$result) {
    pg_query($database, "rollback");
    print "<li>" . err_str($strings['errors']['db_insert_failed']) ."</li>";
    print "<li><pre>$query</pre></li>\n";
    print "<li>".$strings['import']['cleaning_up']."</li>";
    $err = error_reporting(0);
    unlink($hires_name);
    unlink($preview_name);
    unlink($thumb_name);
    error_reporting($err);
    print "</ul>";
    return FALSE;
  }

  pg_query($database, "commit");
  print "</ul>";
  return TRUE;
}

function import_raw_decode_dcraw($input_file_name, $camera_input_profile, $verbose, &$image_data) {
  global $sys_dcraw;
  global $tmp_volume_path;
  global $strings;
  global $po_options;

  $temporary_file_name = tempnam($tmp_volume_path, "po");
  unlink($temporary_file_name);
  $temporary_file_name = $temporary_file_name . ".ppm";

  if (!is_executable($sys_dcraw)) 
    return FALSE;

  if ($verbose) {
    print "<li>".sprintf($strings['import']['decoding_raw_using'], 'dcraw')." ... ";
  }

  $options = " -q 3 -b $po_options[dcraw_brightness]";

  //  $options = " $options -B 2 4"

  switch ($po_options['raw_white_balance']) {
  case "automatic":
    $options = $options . " -a";
    break;
  case "camera":
    $options = $options . " -w";
  }
  switch ($po_options['raw_bitdepth']) {
  case 24:
    break;
  case 48:
    $options = $options . " -4";
  }

  $options = $options . " -o 1"; // output to sRGB colorspace

  if ($camera_input_profile != FALSE) {
    if ($camera_input_profile == "") $camera_input_profile = "embed";
    $options = $options . " -p $camera_input_profile";
  }

  $cmdline = "$sys_dcraw $options -c " . escapeshellarg($input_file_name) . " > $temporary_file_name ";

  /*
  if ($verbose) {
    print "<em>$cmdline</em> ";
  }
  */

  system($cmdline, $retval_decoding);

  if ($retval_decoding) {
    if ($verbose) {
      print "<font color=\"red\">".$strings['generic']['failed'].".</font></li>";
      print "<li><pre>$cmdline</pre></li>";
    }
    return FALSE;
  }

  /* dcraw automatically rotates images */
  $image_data["orientation"] = "Normal (O deg)";

  if ($verbose) {
    print $strings['generic']['done'].".</li>";
  }

  return $temporary_file_name;
}

function import_raw_decode_exiftool($input_file_name, $camera_input_profile, $verbose, &$image_data) {
  global $sys_exiftool;
  global $tmp_volume_path;
  global $strings;

  if (!is_executable($sys_exiftool))
    return FALSE;

  /* If we don't have a RAW preview image, we're SOL here */
  if (!$image_data['jpgfromraw'])
    return FALSE;
  
  /* If we're already extracted the image, just use that one. */
  if (file_exists($image_data['jpgfromraw']))
    return $image_data['jpgfromraw'];

  /* Clear the jpegfromraw data so we don't think we have a secondary
     image to import.  If we actually do, the caller has to fix it up. */
  $image_data['jpgfromraw'] = FALSE;

  $temporary_file_name = tempnam($tmp_volume_path, "po");
  unlink($temporary_file_name);
  $temporary_file_name = $temporary_file_name . ".jpg";

  $options = " -b -JpgFromRaw > " . escapeshellarg($temporary_file_name);

  if ($verbose) {
    print "<li>".sprintf($strings['import']['decoding_raw_using'], 'ExifTool')." ... ";
  }

  $cmdline = "$sys_exiftool " . escapeshellarg($input_file_name) . " $options ";

  /*
  if ($verbose) {
    print "<em>$cmdline</em> ";
  }
  */

  system($cmdline , $retval_decoding);

  if ($retval_decoding) {
    if ($verbose) {
      print "<font color=\"red\">".$strings['generic']['failed'].".</font></li>";
      print "<li><pre>$cmdline</pre></li>";
    }
    return FALSE;
  }

  if ($verbose) {
    print $strings['generic']['done'].".</li>";
    flush();
  }

  return $temporary_file_name;
}

function import_raw_decode_ufraw($input_file_name, $camera_input_profile, $verbose, &$image_data) {
  // global $icc_profiles;
  global $sys_ufraw;
  global $po_options;
  global $tmp_volume_path;
  global $strings;

  if (!is_executable($sys_ufraw))
    return FALSE;

  $options = "";

  if ($camera_input_profile != FALSE && $camera_input_profile != "") {
    $temporary_config_file = tempnam($tmp_volume_path, "po");
       $output_file=fopen($temporary_config_file,"w");
       fwrite($output_file,"<?xml version='1.0' encoding='utf-8'?>\n");
       fwrite($output_file,"<UFRaw Version='7'>\n");
       fwrite($output_file,"<InputProfile Current='yes'>Profile\n");
       fwrite($output_file,"   <File>".$camera_input_profile."</File>\n");
       fwrite($output_file,"</InputProfile>\n");
       fwrite($output_file,"<sRGBOutputProfile Current='yes'>\n");
       fwrite($output_file,"</sRGBOutputProfile>\n");
       fwrite($output_file,"</UFRaw>\n");
       fclose($output_file);
    $options = $options . " --conf=$temporary_config_file";
  }

  $curve = "camera"; // curve == linear/camera/filename
  $exposure = "auto"; // exposure compensation

  $linearity = $po_options['ufraw_gamma_linearity'];
  $gamma = $po_options['ufraw_gamma']; 

  if (($curve == "camera") || ($curve == "linear") || ($curve=="custom")) { 
    $curve = "--base-curve=$curve";
  } else {
    $curve = "--base-curve-file=$curve";
  }

  $temporary_file_name = tempnam($tmp_volume_path, "po");
  unlink($temporary_file_name);
  $temporary_file_name = $temporary_file_name . ".ppm";

  if ($verbose) {
    print "<li>".sprintf($strings['import']['decoding_raw_using'], 'ufraw')." ... ";
  }

  $options = $options . " --exposure=$exposure --gamma=$gamma --linearity=$linearity  --interpolation=ahd $curve";
  switch ($po_options['raw_white_balance']) {
  case "automatic":
    $options = $options . " --wb=auto";
    break;
  case "camera":
    $options = $options . " --wb=camera";
  }
  switch ($po_options['raw_bitdepth']) {
  case 24: 
    $options = $options . " --out-type=ppm8";
    break;
  case 48:
    $options = $options . " --out-type=ppm16";
  }

  $cmdline = "$sys_ufraw $options --silent --output=$temporary_file_name " . escapeshellarg($input_file_name); 

  system($cmdline, $retval_decoding);

  /*
  if ($verbose) {
    print "<em>$cmdline</em> ";
  }
  */

  if ($retval_decoding) {
    if ($verbose) {
      print "<font color=\"red\">".$strings['generic']['failed'].".</font></li>";
      print "<li><pre>$cmdline</pre></li>";
    }
       unlink($temporary_config_file);

    return FALSE;
  }

  /* ufraw automatically rotates images */
  $image_data["orientation"] = "Normal (O deg)";

  if ($verbose) {
    print $strings['generic']['done'].".</li>";
    flush();
  }

  unlink($temporary_config_file);

  return $temporary_file_name;
}

function import_raw_decode_dcm2pnm($input_file_name, $camera_input_profile, $verbose, &$image_data) {
  global $sys_dcm2pnm;
  global $tmp_volume_path;
  global $strings;

  if (!is_executable($sys_dcm2pnm))
    return FALSE;

  $temporary_file_name = tempnam($tmp_volume_path, "po");
  unlink($temporary_file_name);
  $temporary_file_name = $temporary_file_name . ".pnm";

  if ($verbose) {
    print "<li>".sprintf($strings['import']['decoding_raw_using'], 'dcm2pnm')." ... ";
  }

  $options = "";

  switch ($po_options['raw_bitdepth']) {
  case 24: 
    $options = $options . " --write-8-bit-pnm";
    break;
  case 48:
    $options = $options . " --write-16-bit-pnm";
  }

  $cmdline = "$sys_dcm2pnm $options " . escapeshellarg($input_file_name) ." $temporary_file_name ";

  system($cmdline, $retval_decoding);

  /*
  if ($verbose) {
    print "<em>$cmdline</em> ";
  }
  */

  if ($retval_decoding) {
    if ($verbose) {
      print "<font color=\"red\">".$strings['generic']['failed'].".</font></li>";
      print "<li><pre>$cmdline</pre></li>";
    }
    return FALSE;
  }

  if ($verbose) {
    print $strings['generic']['done'].".</li>";
    flush();
  }

  return $temporary_file_name;
}

function import_raw_decode($input_file_name, $decoder, $camera_input_profile, $verbose, &$image_data) {

  global $icc_profiles;

  if (($camera_input_profile == FALSE) ||
      ($camera_input_profile == null) ||
      ($camera_input_profile == "None")) {
    $camera_input_profile = FALSE;
  } else {
    $camera_input_profile = $icc_profiles[$camera_input_profile]['file'];
  }
  
  /* Figure out if we have an embedded preview image to extract */
  if ($image_data['jpgfromraw']) {
    /* This extracts the embedded image to a temporary file */
    $image_data['jpgfromraw'] = import_raw_decode_exiftool($input_file_name, $camera_input_profile, $verbose, $image_data);
  } 

  $res = FALSE;
  $decoders = split(",", $decoder);
  foreach ($decoders as $decoder) {
    $decoder = trim($decoder);
    if ($decoder == "dcraw") {
      $res = import_raw_decode_dcraw($input_file_name, $camera_input_profile, $verbose, $image_data);
    } else if ($decoder == "exiftool") {
      $res = import_raw_decode_exiftool($input_file_name, $camera_input_profile, $verbose, $image_data);
    } else if ($decoder == "ufraw") {
      $res = import_raw_decode_ufraw($input_file_name, $camera_input_profile, $verbose, $image_data);
    } else if ($decoder == "dcm2pnm") {
      $res = import_raw_decode_dcm2pnm($input_file_name, $camera_input_profile, $verbose, $image_data);
    } else {
      $res = FALSE;
    }  
    if ($res != FALSE) break;
  }
  return $res;
}

function import_generate_thumbnail($input_file_name, $thumb_name, $verbose, $multi_page_parameter, $orientation, $colorspace) 
{
  global $sys_convert;
  global $thumb_max_size;
  global $icc_profiles;
  global $strings;
  global $po_options;

  if (($po_options['thumb_border_width'] != 0) && ($po_options['thumb_border_height'] != 0))
    $im_framing_option = " -frame " . escapeshellarg($po_options['thumb_border_width']  ."x". $po_options['thumb_border_height']) . " -mattecolor " . escapeshellarg("#".$po_options['thumb_border_color']);
  else
    $im_framing_option = "";

  $im_profile_a = $im_profile_b = "";

  if ($colorspace != "" && $colorspace > "1") {
    $im_profile_b = " -profile " . $icc_profiles[1]['file'] . " +profile icm";

    if ($colorspace != "99") {
      $im_profile_a = " +profile icm -profile " . $icc_profiles[$colorspace]['file'];
    }
  }

  if ($verbose) {
    print "<li>".$strings['import']['generating_thumb'] ." ... ";
  }

  /* generate thumbnail */
  $cmdline = "$sys_convert ".
    $im_profile_a .
    " -thumbnail " .   escapeshellarg($thumb_max_size  ."x". $thumb_max_size) ."\> ".
    " -unsharp ".     escapeshellarg($po_options['thumb_sharpen_radius']  ."x". $po_options['thumb_sharpen_sigma'] ."+". $po_options['thumb_sharpen_amount'] ."+". $po_options['thumb_sharpen_threshold']) ." ".
    $im_framing_option ." " .
    $orientation ." ".
    $im_profile_b ." ".
    (($po_options['thumb_color_space'] == "RGB") ? "" : " -colorspace " . escapeshellarg($po_options['thumb_color_space']))  ." ".
    (($po_options['thumb_gamma'] == 1) ? "" : " -gamma " .      escapeshellarg($po_options['thumb_gamma'])) ." ".
    " -quality " .    escapeshellarg($po_options['thumb_quality'])  ." ".
    escapeshellarg($input_file_name).escapeshellarg($multi_page_parameter).
    " -strip ". 
    escapeshellarg($thumb_name);

  /*
   if ($verbose) {
   print "<em>$cmdline</em>";
   }
  */
  
  system($cmdline, $retval_thumb_scaling);

  if ($retval_thumb_scaling) {
    if ($verbose) {
      print "<font color=\"red\">".$strings['generic']['failed'].".</font></li>";
      print "<li><pre>$cmdline</pre></li>";
    }
    return FALSE;
  }

  if ($verbose) {
    print $strings['generic']['done'].". ($thumb_max_size ".$strings['generic']['pixels'] .")</li>";
    flush();
  }

  return TRUE;
}

function import_generate_preview($input_file_name, $preview_name, $verbose, $multi_page_parameter, $orientation, $colorspace) {
  global $sys_convert;
  global $sys_composite;
  global $image_max_size;
  global $icc_profiles;
  global $strings;
  global $po_options;
  global $image_repository_path; 


  /* matting options */
  if (($po_options['preview_border_width'] != 0) && ($po_options['preview_border_height'] != 0))
    $im_framing_option = " -frame " . escapeshellarg($po_options['preview_border_width']  ."x". $po_options['preview_border_height']) . " -mattecolor " . escapeshellarg("#".$po_options['preview_border_color']);

  else
    $im_framing_option = "";

  $im_profile_a = $im_profile_b = "";

  if ($colorspace != "" && $colorspace > "1") {
    $im_profile_b = " -profile " . $icc_profiles[1]['file'] . " +profile icm";

    if ($colorspace != "99") {
      $im_profile_a = " +profile icm -profile " . $icc_profiles[$colorspace]['file'];
    }
  }
  
  if ($verbose) {
    print "<li>".$strings['import']['generating_preview'] ." ... ";
  }

  /* generate preview */
  $cmdline = "$sys_convert ".
    $im_profile_a .
    " -resize " .   escapeshellarg($image_max_size ."x". $image_max_size) ."\> ".
    " -unsharp ".     escapeshellarg($po_options['preview_sharpen_radius']  ."x". $po_options['preview_sharpen_sigma'] ."+". $po_options['preview_sharpen_amount'] ."+". $po_options['preview_sharpen_threshold']) ." ".
    $im_framing_option ." ".
    $orientation ." ".
    $im_profile_b ." ".
    (($po_options['preview_color_space'] == "RGB") ? "" : " -colorspace " . escapeshellarg($po_options['preview_color_space']))  ." ".
    (($po_options['preview_gamma'] == 1) ? "" : " -gamma " .      escapeshellarg($po_options['preview_gamma'])) .
    " -quality " . 
    escapeshellarg($po_options['preview_quality']) ." ".
    escapeshellarg($input_file_name).escapeshellarg($multi_page_parameter).
    " -strip ". 
    escapeshellarg($preview_name);
  
  /*
  if ($verbose) {
    print "<em>$cmdline</em>";
  }
  */
  system($cmdline, $retval_image_scaling);

  if ($verbose) {
    print $strings['generic']['done'].". ($image_max_size ".$strings['generic']['pixels'] .")</li>";
  }
  
  /* Are we supposed to watermark the preview image? */
  if ($po_options['watermark_location'] && 
      $po_options['watermark_brightness'] && 
      $po_options['watermark_photo']) {

    /* Fix up the watermark path as needed */
    if (substr($po_options['watermark_path'], 0, 1) != '/') {
      $po_options['watermark_path'] = "$image_repository_path/$po_options[watermark_path]";
    }
    
    if ($verbose) {
      print "<li>".$strings['import']['watermarking'] ." ... ";
    }

    $cmdline = "$sys_composite".
      " -gravity " .  escapeshellarg($po_options['watermark_location']) .
      " -watermark " .  escapeshellarg($po_options['watermark_brightness']) ." ".
      escapeshellarg($po_options['watermark_path']) ." ".
      escapeshellarg($preview_name)." ".
      escapeshellarg($preview_name);

    /*
    if ($verbose) {
      print "<em>$cmdline</em>";
    }
    */

    system($cmdline, $retval_watermarking);
        
    if ($retval_watermarking) {
      if ($verbose) {
	print "<font color=\"red\">".$strings['generic']['failed'].".</font></li>";
	print "<li><pre>$cmdline</pre></li>";
      }
      return FALSE;
    }

    if ($verbose) {
      print $strings['generic']['done'].".</li>";
      flush();
    }
  }
    
  return TRUE;
}

function file_upload(&$image_data, $index, $hires_name, $preview_name, $thumb_name, $multi_page_parameter, $decoder, $database) {
  global $strings;
  global $po_options;

  $success=TRUE;

  if ($image_data['file'][$index]['name_tmp']) {

    $original_name = $image_data['file'][$index]['original_name'];

    print "<li>" . sprintf($strings['import']['copying_into_repo'], $original_name) . " ... ";
    /* copy the original to the repository */
    if (!copy($image_data['file'][$index]['name_tmp'], $hires_name)) {
      print "<font color=\"red\">".$strings['generic']['failed'].".</font></li>";
      print "<li><pre>cp ".$image_data['file'][$index]['name_tmp']." $hires_name</pre></li>";
      return FALSE;
    }
    print $strings['generic']['done']."</li>";
    flush();

    /* Raw Decode */
    $temporary_ppm_file = import_raw_decode($image_data['file'][$index]['name_tmp'], $decoder, $image_data['camera_input_profile'], TRUE, $image_data);
    if ($temporary_ppm_file) {
      unlink($image_data['file'][$index]['name_tmp']);
      $image_data['file'][$index]['name_tmp'] = $temporary_ppm_file;
    }

    /* raw decode can alter orientation */
    $orientation = orientation_id_from_string($database, $image_data["orientation"]);
    $image_data["orientation_xform"] = orientation_to_xform($database, $orientation);

    // print "<li>$image_data[orientation] => $orientation => $image_data[orientation_xform]</li>";

    /* Preview */
    $input_file_name = $preview_name;
    $return_value = import_generate_preview($image_data['file'][$index]['name_tmp'], $input_file_name, TRUE, $multi_page_parameter, $image_data["orientation_xform"], $image_data['colorspace']);
    if (!$return_value) {
      unlink($input_file_name);
      $success=false;
    }

    if ($success && ($po_options['preview_copy_metadata'] == 't')) {
      $success = transfer_metadata($image_data, $hires_name, $preview_name, TRUE, TRUE);
    }

    /* Thumbnails */
    if ($success) {
      $input_file_name = $thumb_name;
      $return_value = import_generate_thumbnail($image_data['file'][$index]['name_tmp'], $input_file_name, TRUE, $multi_page_parameter, $image_data["orientation_xform"], $image_data['colorspace']);
      if (!$return_value) {
	unlink($input_file_name);
	$success=false;
      }
    }

    /* remove temporary file */
    unlink($image_data['file'][$index]['name_tmp']);
  }

  return $success;
}

?>
