--  Copyright (C) 2007 Solomon Peachy (pizza@shaftnet.org)

--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 3 of the License, or
--  (at your option) any later version.

--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.

--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

----------------------------------------------------------
-- v2.33 to v2.34 patch
----------------------------------------------------------

-- Drop obselete user preferences

alter table preferences drop num_of_rows;
alter table preferences drop num_of_columns;
alter table preferences drop num_of_rows_l;

-- Drop more obselete user stuff

alter table users drop shared;
alter table users drop session_id;
alter table system_preferences drop new_user_shared;

-- Add the 'Spool' Album for everyone.

insert into album (identifier, parent_album, users, caption, type,
                    date_of_creation, access_rights, orderby, description)
  select nextval('album_id_sequence') as identifier,
         null as parent_album,
         identifier as users,
         'Spool' as caption,
	 'u' as type,
         now() as date_of_creation,
         3 as access_rights,
         7 as orderby,
         null as description
    from users;

-- Drop the old spooler

drop table photo_spooler;
drop sequence photo_spooler_id_sequence;

-- Additional indices

create index photo_dupe_key on photo_dupe(key);

-- GPS information...

alter table photo_tech add latitude numeric;
alter table photo_tech add longitude numeric;
alter table photo_tech add altitude numeric;
alter table photo_tech add img_direction numeric;

-- Fix this broken preference..

update preferences set search_default_mode = 1 where search_default_mode is null;
update preferences set folder_view = 'slide' where folder_view is null;

-- Add password support to folders/albums

alter table folder add password text;
alter table album add password text;

-----  Recursive bulk updates!

CREATE OR REPLACE FUNCTION folder_and_subfolders(int) RETURNS SETOF int AS '
DECLARE
   ident record;
   ident2 record;
BEGIN
   RETURN NEXT $1;
   FOR ident IN SELECT identifier FROM folder f WHERE f.parent_folder = $1 LOOP
       FOR ident2 IN SELECT * FROM folder_and_subfolders(ident.identifier) LOOP
         RETURN NEXT ident2.folder_and_subfolders;
        END LOOP;
   END LOOP;

   RETURN;
END;
' LANGUAGE 'plpgsql';

CREATE OR REPLACE FUNCTION album_and_subalbums(int) RETURNS SETOF int AS '
DECLARE
   ident record;
   ident2 record;
BEGIN
   RETURN NEXT $1;
   FOR ident IN SELECT identifier FROM album f WHERE f.parent_album = $1 LOOP
       FOR ident2 IN SELECT * FROM album_and_subalbums(ident.identifier) LOOP
         RETURN NEXT ident2.album_and_subalbums;
        END LOOP;
   END LOOP;

   RETURN;
END;
' LANGUAGE 'plpgsql';

------ key=value user preferences

create table user_preferences (
	owner integer not null references users(identifier),
	key varchar(100) not null,
	value text
);

create index user_preferences_owner_idx on user_preferences(owner);

CREATE OR REPLACE FUNCTION get_user_pref(int, text) RETURNS text AS '
  SELECT value 
           FROM user_preferences 
          WHERE owner = $1  
            AND key = $2;
' LANGUAGE 'sql';

-- Once we're done migrating:

-- ALTER TABLE users DROP preferences;
-- ALTER TABLE users DROP imagemagick_options;
-- ALTER TABLE users DROP imagemagick_composite_options;
-- ALTER TABLE users DROP dcraw_options;

-- DROP TABLE imagemagick_options;
-- DROP TABLE imagemagick_composite_options;
-- DROP TABLE dcraw_options;
-- DROP TABLE preferences;
-- DROP SEQUENCE imagemagick_options_id_sequence;
-- DROP SEQUENCE imagemagick_composite_options_id_sequence;
-- DROP SEQUENCE dcraw_options_id_sequence;
-- DROP SEQUENCE preferences_id_sequence;

------ Drop system preferences table too!

drop table system_preferences;

------ Add a field to track when a photo was added

alter table photo add date_added timestamp without time zone;

------ Add 'version' to album_content to allow individual versions.

ALTER TABLE album_content
  ADD version integer REFERENCES photo_version(identifier);

UPDATE album_content set version =
   (SELECT identifier from photo_version v
    WHERE album_content.photo = v.photo AND v.master = 't');

ALTER TABLE album_content ALTER version SET NOT NULL;

create index album_content_version_idx on album_content(version);

---- Drop useless indices!

drop index album_content_id_idx;
drop index photo_id_idx;
drop index folder_id_idx;
drop index album_id_idx;

---- Add 'last modified' timestamp to everything

alter table photo add date_changed timestamp without time zone;
alter table folder add date_changed timestamp without time zone;
alter table album add date_changed timestamp without time zone;
alter table photo_version add date_changed timestamp without time zone;

alter table album_content add date_added timestamp without time zone;

-------- Drop shopping infrastructure

drop view view_shop_item_generator_rule ;
drop table shop_order;
drop table shop_transaction;
drop table shop_transaction_status;
drop table shop_item_generator_rule;
drop table photo_item;
drop table shop_item;
drop table shop_category;
drop table currency;

drop function is_photo_for_sale(integer);

drop sequence photo_item_id_sequence; 
drop sequence shop_category_id_sequence;
drop sequence shop_item_generator_rule_id_sequence;
drop sequence shop_item_id_sequence;
drop sequence shop_order_id_sequence;
drop sequence shop_transaction_id_sequence;
drop sequence shop_transaction_status_id_sequence;
drop sequence currency_id_sequence;

------ Add 'version' to rating to allow individual versions.

ALTER TABLE rating
  ADD version integer REFERENCES photo_version(identifier);

UPDATE rating set version =
   (SELECT identifier from photo_version v
    WHERE rating.photo = v.photo AND v.master = 't');

ALTER TABLE rating ALTER version SET NOT NULL;

create index rating_version_idx on rating(version);

DROP VIEW view_rating;

create view view_rating as
  select rating.identifier, photo, users, date_of_creation, value, first_name, last_name, comment, version
    from rating, users
   where rating.users=users.identifier;

----  Add a table to keep track of individual image files.

create table files (
	identifier integer primary key,
	version integer references photo_version(identifier) not null,
	size integer not null, 
	x_res integer not null,
	y_res integer not null,
	path text not null,
	created timestamp without time zone not null,
	filesize integer not null,
	params text not null,
	watermarked boolean not null,
	comments text
);

create index files_version_idx on files(version);

create sequence files_id_sequence start 1;

-- alter table photo_version drop small_image_path;
-- alter table photo_version drop medium_image_path;
-- alter table photo_version drop large_image_path;

CREATE OR REPLACE FUNCTION get_image_path(int, int) RETURNS text AS '
  SELECT path 
           FROM files
          WHERE version = $1  
            AND size = $2;
' LANGUAGE 'sql';

---- Additional indices

create index photo_version_photo on photo_version(photo);

-------- Drop submission infrastructure

drop view view_submission;
drop table submission;
drop table submission_status;

------ Add 'version' to photo_dupe to allow individual versions.

ALTER TABLE photo_dupe
  ADD version integer REFERENCES photo_version(identifier);

UPDATE photo_dupe set version =
   (SELECT identifier from photo_version v
    WHERE photo_dupe.photo = v.photo AND v.master = 't');

ALTER TABLE photo_dupe ALTER version SET NOT NULL;

create index photo_dupe_version_idx on photo_dupe(version);

---- Fix up a couple of bogus functions

create or replace function count_equipment_by_user(integer) returns bigint as '
	select ((select count(identifier) from lens where users = $1) +
            (select count(identifier) from camera where users = $1) +
            (select count(identifier) from scanner where users = $1) +
            (select count(identifier) from film where users = $1) +
            (select count(identifier) from support where users = $1) +
            (select count(identifier) from flash where users = $1))
' language 'sql';

create or replace function count_equipment_by_user(integer, integer) returns bigint as '
	select ((select count(identifier) from lens where users = $1 and access_rights <= $2) +
            (select count(identifier) from camera where users = $1 and access_rights <= $2) +
            (select count(identifier) from scanner where users = $1 and access_rights <= $2) +
            (select count(identifier) from film where users = $1 and access_rights <= $2) +
            (select count(identifier) from support where users = $1 and access_rights <= $2) +
            (select count(identifier) from flash where users = $1 and access_rights <= $2))
' language 'sql';

create or replace function number_of_manufacturer_ref(integer) returns bigint as '
	select ((select count(identifier) from lens_type where manufacturer = $1) +
            (select count(identifier) from camera_type where manufacturer = $1) +
            (select count(identifier) from scanner_type where manufacturer = $1) +
            (select count(identifier) from support_type where manufacturer = $1) +
            (select count(identifier) from film_type where manufacturer = $1) +
            (select count(identifier) from flash_type where manufacturer = $1))
' language 'sql';

----- Add a flag to ignore the camera's user comments

alter table camera add ignore_comment boolean;
update camera set ignore_comment = 'f';
alter table camera alter ignore_comment set not null;

drop view view_camera;
create view view_camera as
  select camera.identifier, manufacturer.name, model, variation,
camera.serial_number, camera.users, date_of_purchase,
camera.purchased_new,
manufacturer.url, (access_type.value) as access_rights, raw_icc_profile, ignore_comment
    from camera_type, camera, manufacturer, access_type
   where camera_type.manufacturer = manufacturer.identifier
     and camera_type.identifier = camera.type
     and camera.access_rights = access_type.identifier
   order by manufacturer.name, model, variation;

---- Add 'lens_id' to the lens types.

alter table lens_type add exiftool_lens_id text;

drop view view_lens;
drop view view_lens_type;

create view view_lens as
  select lens.identifier, manufacturer.name, model, variation,
        s_min_aperture, s_max_aperture, s_min_focal_length,
        s_max_focal_length, lens.serial_number, lens.users,
        date_of_purchase, lens.purchased_new,
        manufacturer.url, (access_type.value) as access_rights,
	exiftool_lens_id
    from lens_type, lens, manufacturer, access_type,
         (select identifier as identifier, value as s_min_focal_length
                from focal_length) as t_min_focal_length,
         (select identifier as identifier, value as s_max_focal_length
                from focal_length) as t_max_focal_length,
         (select identifier as identifier, value as s_min_aperture from
                aperture) as t_min_aperture,
         (select identifier as identifier, value as s_max_aperture from
                aperture) as t_max_aperture
   where lens_type.manufacturer = manufacturer.identifier
     and lens_type.identifier = lens.type
     and lens_type.min_focal_length = t_min_focal_length.identifier
     and lens_type.max_focal_length = t_max_focal_length.identifier
     and lens_type.min_aperture = t_min_aperture.identifier
     and lens_type.max_aperture = t_max_aperture.identifier
     and lens.access_rights = access_type.identifier
   order by manufacturer.name, s_min_focal_length, model, variation;

create view view_lens_type as select lens_type.identifier, manufacturer.name,
        model, variation, s_min_aperture, s_max_aperture,
        s_min_focal_length, s_max_focal_length, manufacturer.url,
        exiftool_lens_id
    from lens_type, manufacturer,
         (select identifier as identifier, value as s_min_focal_length
        from focal_length) as t_min_focal_length,
         (select identifier as identifier, value as s_max_focal_length
        from focal_length) as t_max_focal_length,
         (select identifier as identifier, value as s_min_aperture from
        aperture) as t_min_aperture,
         (select identifier as identifier, value as s_max_aperture from
        aperture) as t_max_aperture
   where lens_type.manufacturer = manufacturer.identifier
     and lens_type.min_focal_length = t_min_focal_length.identifier
     and lens_type.max_focal_length = t_max_focal_length.identifier
     and lens_type.min_aperture = t_min_aperture.identifier
     and lens_type.max_aperture = t_max_aperture.identifier
   order by manufacturer.name, s_min_focal_length, model, variation;

--- A few misc things.

drop index users_idx;
create index folder_parent_idx on folder(parent_folder);
create index photo_folder on photo(folder);

--- Add a 'hide' field to the users table.

alter table users add hide boolean;
alter table users alter column hide set default 'f';
update users set hide = 'f';
alter table users alter column hide set not null;

--- Drop 'orderby' fields; they're useless now.

alter table folder drop orderby;
alter table album drop orderby;

--- Add a 'purcahse url' to photos.

alter table photo add store_url text;

--- Permission lookup functions!

CREATE OR REPLACE FUNCTION can_access_photo(integer, integer, varchar[]) returns boolean as '
DECLARE
  res_id alias for $1;
  user_id alias for $2;
  passwd_list alias for $3;
  res_access integer;
  owns_id integer;
  can_access boolean;
  admin_type boolean;
BEGIN

  -- 1 == public, 2 == protected, 3 == private
  -- 1 == diabled, 2 == client, 3 == user, 4 == admin

SELECT INTO res_access, owns_id access_rights, users FROM photo WHERE identifier = res_id;

IF res_access = 1 THEN
  RETURN true;
END IF;

IF owns_id = user_id THEN
  RETURN true;
END IF;

SELECT INTO admin_type true FROM users WHERE identifier = user_id AND type = 4;
IF admin_type is true THEN
  RETURN true;
END IF;

IF res_access = 2 THEN
  SELECT INTO can_access true FROM photo, folder, calendar WHERE folder.identifier = photo.folder AND folder.event = calendar.identifier AND calendar.client = user_id;
  IF can_access is true THEN
    RETURN true;
  END IF;

  SELECT INTO can_access true FROM photo, folder WHERE photo.identifier = res_id AND photo.folder = folder.identifier AND folder.password = ANY(passwd_list);
  IF can_access is true THEN
    RETURN true;
  END IF;

  SELECT INTO can_access true FROM album_content, album, calendar WHERE album_content.photo = res_id AND album.identifier = album_content.album AND album.event = calendar.identifier AND calendar.client = user_id;
  IF can_access is true THEN
    RETURN true;
  END IF;

  SELECT INTO can_access true FROM album, album_content WHERE album_content.photo = res_id AND album_content.album = album.identifier AND album.password = ANY(passwd_list);
  IF can_access is true THEN
    RETURN true;
  END IF;

END IF;

  return false;
END;
' language 'plpgsql';

CREATE OR REPLACE FUNCTION can_access_folder(integer, integer, varchar[]) returns boolean as '
DECLARE
  res_id alias for $1;
  user_id alias for $2;
  passwd_list alias for $3;
  res_access integer;
  owns_id integer;
  can_access boolean;
  admin_type boolean;
  passwd varchar;
BEGIN

  -- 1 == public, 2 == protected, 3 == private
  -- 1 == diabled, 2 == client, 3 == user, 4 == admin

SELECT INTO res_access, owns_id, passwd access_rights, users, password FROM folder WHERE identifier = res_id;

IF res_access = 1 THEN
  RETURN true;
END IF;

IF owns_id = user_id THEN
  RETURN true;
END IF;

SELECT INTO admin_type true FROM users WHERE identifier = user_id AND type = 4;
IF admin_type is true THEN
  RETURN true;
END IF;

IF res_access = 2 THEN
  SELECT INTO can_access true FROM folder, calendar WHERE folder.event = calendar.identifier AND calendar.client = user_id;

  IF can_access is true THEN
   RETURN true;
  END IF;

  select into can_access true WHERE passwd = ANY(passwd_list);
  IF can_access is true THEN
    RETURN true;
  END IF;
END IF;

  return false;
END;
' language 'plpgsql';


CREATE OR REPLACE FUNCTION can_access_album(integer, integer, varchar[]) returns boolean as '
DECLARE
  res_id alias for $1;
  user_id alias for $2;
  passwd_list alias for $3;
  res_access integer;
  owns_id integer;
  can_access boolean;
  admin_type boolean;
  passwd varchar;
BEGIN

  -- 1 == public, 2 == protected, 3 == private
  -- 1 == diabled, 2 == client, 3 == user, 4 == admin

SELECT INTO res_access, owns_id, passwd access_rights, users, password FROM album WHERE identifier = res_id;

IF res_access = 1 THEN
  RETURN true;
END IF;

IF owns_id = user_id THEN
  RETURN true;
END IF;

SELECT INTO admin_type true FROM users WHERE identifier = user_id AND type = 4;
IF admin_type is true THEN
  RETURN true;
END IF;

IF res_access = 2 THEN
  SELECT INTO can_access true FROM album, calendar WHERE album.event = calendar.identifier AND calendar.client = user_id;

  IF can_access is true THEN
     RETURN true;
  END IF;

  select into can_access true WHERE passwd = ANY(passwd_list);
  IF can_access is true THEN
    RETURN true;
  END IF;
END IF;

  return false;
END;
' language 'plpgsql';


--- Replace functions that to use the new permission functions

drop function count_subfolders_by_folder(integer);
drop function count_subfolders_by_folder(integer, integer);

create or replace function count_subfolders_by_folder(integer, integer, varchar[]) returns bigint as '
select count(identifier) from folder where parent_folder = $1 and can_access_folder(identifier, $2, $3);
' language 'sql';

drop function count_photos_by_folder(integer);
drop function count_photos_by_folder(integer, integer);

create or replace function count_photos_by_folder(integer, integer, varchar[]) returns bigint as '
select count(identifier) from photo where folder = $1 and can_access_photo(identifier, $2, $3);
' language 'sql';

drop function count_subalbums_by_album(integer);
drop function count_subalbums_by_album(integer, integer);

create or replace function count_subalbums_by_album(integer, integer, varchar[]) returns bigint as '
select count(identifier) from album where parent_album = $1 and can_access_album(identifier, $2, $3);
' language 'sql';

drop function count_photos_by_album(integer);
drop function count_photos_by_album(integer, integer);

create or replace function count_photos_by_album(integer, integer, varchar[]) returns bigint as '
select count(photo) from album_content where album = $1 and can_access_photo(photo, $2, $3);
' language 'sql';

---- Add an 'rdf' column to store all XMP data.

alter table photo_tech add rdf text;

---- Track clients with datebook entries..

alter table calendar add client integer references users(identifier);
create index calendar_client_idx on calendar(client);
create index calendar_user_idx on calendar(users);

---- And also associate events with folders/albums

alter table album add column event integer references calendar(identifier);
alter table folder add column event integer references calendar(identifier);
create index album_event_idx on album(event);
create index folder_event_idx on folder(event);

---- this is obselete		     
alter table album drop column type;

---- Add a comments column to the photos.
alter table photo add column comments text;

---- Update the files table to be smarter about watermarks
alter table files drop column watermarked;
alter table files add column watermark integer references photo_version(identifier);
